/*
 * Memory Setup stuff - taken from Cirrus Logic EP9312 spec Revision 0.14
 *
 * Copyright (C) 2002 2003  Adam Bezanson, Network Audio Technologies, Inc.
 *           (bezanson@netaudiotech.com)
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */



#include <config.h>
#include <version.h>
#include <ep93xx.h>

/* Include SDRAM stuff */
#include "sdramcfg.S"

	
	
.globl lowlevel_init
lowlevel_init:

        // 
        // Make sure caches are off
        //
        ldr     r1,=0
        mcr     p15,0,r1,c1,c0,0
        nop
        nop
        nop
        nop

        //
        // Go back to running from the external clock.
        //
        ldr     r1, =EP9312_CLKSET1
        mov     r0, #0
        str     r0, [r1]
        nop
        nop
        nop
        nop
        nop

        // 
        // Turn off both LEDs
        // 
        ldr     r1, =EP9312_LED_DATA
        ldr     r0, [r1]

        // 
        // Turn on both LEDs
        //
        orr     r0, r0, #(EP9312_LED_GREEN_ON | EP9312_LED_RED_ON)
        str     r0, [r1]                                       

        //
        // Turn off auto-precharge.
        //
        ldr     r1, =EP9312_SDRAMCTRL
        ldr     r0, [r1,#EP9312_SDRAMCTRL_DEVCFG_3]
        bic     r0, r0, #EP9312_SDRAMCTRL_DEVCFG_AUTOPRE
        str     r0, [r1,#EP9312_SDRAMCTRL_DEVCFG_3]
            
        // 
        // Turn off the Green LED and leave the Red LED on.
        //
        ldr     r1, =EP9312_LED_DATA
        ldr     r0, [r1]
        bic     r0, r0, #(EP9312_LED_GREEN_ON)
        str     r0, [r1]


        //
        // Need to make sure that SDRAM is configured correctly before
        // coping the code into it.
        //
        SdramCfg
            
        nop
        nop
        nop
        nop

        // 
        // Turn on Green LED, Turn off the Red LED.
        //
        ldr     r1, =EP9312_LED_DATA
        ldr     r0, [r1]
        eor     r0, r0, #(EP9312_LED_GREEN_ON | EP9312_LED_RED_ON)
        strb    r0,[r1]                                     
				
		mov pc,lr
