/*
 * Memory Setup stuff - taken from Cirrus Logic EP9312 spec Revision 0.14
 *
 * Copyright (C) 2002 2003  Adam Bezanson, Network Audio Technologies, Inc.
 *           (bezanson@netaudiotech.com)
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */



#include <config.h>
#include <version.h>
#include <ep93xx.h>

/* Include SDRAM stuff */
#include "sdramcfg.S"

.globl lowlevel_init
lowlevel_init:
	mov	r3, #0x80000000		/* I/O base */

	/* GPIO */
	orr	r0, r3, #0x840000
	mov	r1, #0x12
	str	r1, [r0, #0x04] /* PBDR  0x02:RY_ON 0x10:BBACKLIGHT*/
	mov	r1, #0x32
	str	r1, [r0, #0x14] /* PBDDR 0x32 */
	
#ifdef CONFIG_HW_WATCHDOG
	mov	r1, #0x40	/* Watch dog start */
#else
	mov	r1, #0x00
#endif
	str	r1, [r0, #0x40] /* PHDR  */
	mov	r1, #0x40
	str	r1, [r0, #0x44] /* PHDDR */

	/* VIC */	
	ldr     r0, = VIC1_BASE
	ldr	r1, = 0xFFFFFFFF
	mov	r2, #0x0
	str	r2, [r0, #0x10] /* INT_ENABLE	*/
        str     r1, [r0, #0x14] /* INT_EN_CLEAR	*/
	ldr     r0, = VIC2_BASE
	str	r2, [r0, #0x10] /* INT_ENABLE	*/
        str     r1, [r0, #0x14] /* INT_EN_CLEAR	*/

	/* DMA */
	ldr     r0, = DMA_BASE
	mov	r1, #0x0
	str	r1, [r0, #0x00] /* DMAMP_TX_0_CONTROL	*/
	str	r1, [r0, #0x40] /* DMAMP_RX_1_CONTROL	*/
	str	r1, [r0, #0x80] /* DMAMP_TX_2_CONTROL	*/
	str	r1, [r0, #0xC0] /* DMAMP_RX_3_CONTROL	*/
	str	r1, [r0, #0x100] /* DMAMM_0_CONTROL	*/
	str	r1, [r0, #0x140] /* DMAMM_1_CONTROL	*/
	str	r1, [r0, #0x240] /* DMAMP_TX_4_CONTROL	*/
	str	r1, [r0, #0x200] /* DMAMP_RX_5_CONTROL	*/
	str	r1, [r0, #0x2C0] /* DMAMP_TX_6_CONTROL	*/
	str	r1, [r0, #0x280] /* DMAMP_RX_7_CONTROL	*/
	str	r1, [r0, #0x340] /* DMAMP_TX_8_CONTROL	*/
	str	r1, [r0, #0x300] /* DMAMP_RX_9_CONTROL	*/

	ldr     r0, = MAC_BASE
	mov	r1, #0x1
	str	r1, [r0, #0x20] /* MAC_SELFCTL	*/
	
	ldr     r0, = USB_BASE
	mov	r1, #0x1
	str	r1, [r0, #0x08] /* HCCOMMANDSTATUS	*/

	ldr     r0, = IRDA_BASE
	mov	r1, #0x0
	str	r1, [r0, #0x18] /* IR_ENABLE	*/

	/* UART */
	ldr     r0, = EP9312_UART1
	mov	r1, #0x0
	str	r1, [r0, #0x14] /* UART1CR	*/
	ldr     r0, = EP9312_UART2
	str	r1, [r0, #0x14] /* UART2CR	*/
	ldr     r0, = EP9312_UART3
	str	r1, [r0, #0x14] /* UART3CR	*/

	/* I2S */
	ldr     r0, = I2S_BASE
	mov	r1, #0x0
	str	r1, [r0, #0x34] /* I2STX0En	*/
	str	r1, [r0, #0x64] /* I2SRX0En	*/
	str	r1, [r0, #0x38] /* I2STX1En	*/
	str	r1, [r0, #0x68] /* I2SRX1En	*/
	str	r1, [r0, #0x3C] /* I2STX2En	*/
	str	r1, [r0, #0x6C] /* I2SRX2En	*/
	
	/* AC97 */
	ldr     r0, = AC97_BASE
	mov	r1, #0x0
	str	r1, [r0, #0x9C] /* AC97GCR	*/

	/* SSP */
	ldr     r0, = EP9307_SSP
	mov	r1, #0x0
	str	r1, [r0, #0x04] /* SSPCR1	*/

	/* Raster */
	ldr     r0, = RASTER_BASE
	mov	r1, #0x0
	str	r1, [r0, #0x24] /* VIDEOATTRIBS	*/

	/* TOUCH */
	ldr     r0, = TOUCH_BASE
	mov	r1, #0x0
	str	r1, [r0, #0x00] /* TSSetup	*/

	/* IDE */
	ldr     r0, = IDE_BASE
	mov	r1, #0x0
	str	r1, [r0, #0x04] /* IDECFG	*/

	/* GPIO */
	ldr     r0, = GPIO_BASE
	mov	r1, #0x00
	str	r1, [r0, #0x9c] /* GPIOAIntEn */
	str	r1, [r0, #0xB8] /* GPIOBIntEn */
	str	r1, [r0, #0x58] /* GPIOFIntEn */
	str	r1, [r0, #0xC8] /* GPIOEEDRIVE */

	bic	r2, pc, #0x0000FF00
	bic	r2, r2, #0x000000FF
        cmp     r2, #0x60000000
	bne     2f
#if 0
	ldr     r0, = EP9312_SYSCON 
        ldr     r1, [r0]
        tst     r1, #0x400
	bne     1f
#else
	b	1f
#endif

2:
#if 0
	/* SYSCON */
	ldr     r0, = EP9312_SYSCON
	mov	r1, #0x00
	mov	r2, #0xaa
	str	r2, [r0, #0xc0]
        str     r1, [r0, #0x84]	/* VIDDIV */
	str	r2, [r0, #0xc0]
        str     r1, [r0, #0x90]	/* KTDIV */
	str	r2, [r0, #0xc0]
        str     r1, [r0, #0x88]	/* MIRDIV */
	str	r2, [r0, #0xc0]
        str     r1, [r0, #0x8C]	/* I2SDIV */
	str	r2, [r0, #0xc0]
        str     r1, [r0, #0x04]	/* PWRCNT */
	str	r2, [r0, #0xc0]
        str     r1, [r0, #0x80]	/* DEVCFG */
#endif
	/* reset cpu */
	ldr     r0, = EP9312_SYSCON 
	mov	r2, #0xaa
	str	r2, [r0, #0xc0]
        ldr     r1, [r0, #0x80]
        orr     r1, r1, #0x80000000
        str     r1, [r0, #0x80]

	str	r2, [r0, #0xc0]
        bic     r1, r1, #0x80000000
        str     r1, [r0, #80]
	
1:	//
        // Go back to running from the external clock.
        //
	ldr     r1, = EP9312_SYSCON
        mov     r0, #0
        str     r0, [r1, #0x20] /* EP9312_CLKSET1 */
        nop
        nop
        nop
        nop
        nop
            
	/* Setting up flash at CS6 0x2A004d23*/
	orr	r0, r3, #0x080000
#if 1
	ldr	r1, =0x2A004d23
	str	r1, [r0, #0x18]
#else
	ldr     r1, =0x0000fbe0
	ldr     r2, [r0, #0x18]
	orr     r2, r2, r1
	str     r2, [r0, #0x18]
#endif	

        //
        // Turn off auto-precharge.
        //
        ldr     r1, =EP9312_SDRAMCTRL
        ldr     r0, [r1,#EP9312_SDRAMCTRL_DEVCFG_3]
        bic     r0, r0, #EP9312_SDRAMCTRL_DEVCFG_AUTOPRE
        str     r0, [r1,#EP9312_SDRAMCTRL_DEVCFG_3]

        // 
        // Turn off the Green LED and leave the Red LED on.
        //
        ldr     r1, =EP9312_LED_DATA
        mov     r0, #(EP9312_LED_RED_ON)
        str     r0, [r1]


        //
        // Need to make sure that SDRAM is configured correctly before
        // coping the code into it.
        //
        SdramCfg
        nop
        nop
        nop
        nop

        // 
        // Turn on Green LED, Turn off the Red LED.
        //
        ldr     r1, =EP9312_LED_DATA
        mov     r0, #(EP9312_LED_GREEN_ON)
        strb    r0,[r1]                                     
				

	/* Setting up CLKSET1 */
	orr	r0, r3, #0x930000
#if CFG_PLL1_FREQ == 368
	ldr	r1, =0x03a4a24F
#else
	ldr	r1, =0x03a4a3d7
#endif
	str	r1, [r0, #0x20]
	nop
	nop
	nop
	nop
	nop

	/* Setting up CLKSET2 */
	mov	r1, #0x30000000
	orr	r1, r1, #0x0d0000
	orr	r1, r1, #0xc300
	orr	r1, r1, #0x17
	str	r1, [r0, #0x24]
	nop
	nop
	nop
	nop
	nop
	
	/* Go to Async mode */
	mrc	p15, 0, r1, c1, c0, 0
	orr	r1, r1, #0xc0000000
	mcr	p15, 0, r1, c1, c0, 0
	mov	r1, #0x11000
3:	subs	r1, r1, #1
	bne	3b

	b	init_vector
vector_data:
	ldr	pc,	=0x60000004

	/* copy exception vector */
init_vector:
	ldr	r0, vector_data	
	mov	r1, #0			/* r1 dest */
	mov	r2, #0x50		/* r2 size of vector            */
copy_loop:
	str	r0, [r1]
	add	r1, r1, #4
	cmp	r1, r2
	blt	copy_loop

	/* wait for PLL2 lock */
        ldr     r1, =0x80930000
4:
        ldr     r2, [r1]
        tst     r2, #0x100
        beq     4b
	
	/* seting up  DevConfig */
        ldr     r0, =0x80930000
	mov	r2, #0xaa
	str	r2, [r0, #0xc0]
	ldr	r1, =0x0B140D10
	str	r1, [r0, #0x80]

#if 0
/* debug */
        ldr     r0, =0x80840000
	mov	r1, #0x01
	str	r1, [r0, #0x08] /* PCDR */
	mov	r1, #0x00
	str	r1, [r0, #0x0C] /* PDDR */
	str	r1, [r0, #0x38] /* PGDR */
	str	r1, [r0, #0x40] /* PHDR */

	mov	r1, #0xFF
	str	r1, [r0, #0x18] /* PCDDR */
	str	r1, [r0, #0x1C] /* PDDDR */
	mov	r1, #0x05
	str	r1, [r0, #0x3C] /* PGDDR */
	mov	r1, #0x7C
	str	r1, [r0, #0x44] /* PHDDR */
/* debug */
#endif
	
	mov pc,lr
