/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
#include <common.h>
#include <ep93xx.h>
#include <i2c.h>
#include <asm/mach-types.h>
#include <linux/byteorder/swab.h>
#include <rtc.h>

extern int sflash_print_info(void);
extern unsigned long sflash_init(void);

/* ------------------------------------------------------------------------- */

/*
 * static inline void delay (unsigned long loops)
 * {
 *	__asm__ volatile ("1:\n"
 *	 "subs %0, %1, #1\n"
 *	 "bne 1b":"=r" (loops):"0" (loops));
 * }
 */

/*
 * Miscellaneous platform dependent initialisations
 */

int board_init (void)
{
    DECLARE_GLOBAL_DATA_PTR;

    /*
    icache_enable();
    */
#ifdef USE_920T_MMU
    dcache_enable();
#endif
#ifdef CONFIG_TB0319_TM
    GPIO_PADR = 0x00;
    GPIO_PADDR = 0x00;
    GPIO_PBDR = 0x12; /* RY_ON BBACKLIGHT*/
    GPIO_PBDDR = 0x32;
    GPIO_PCDR = 0x00;
    GPIO_PCDDR = 0xFF;
    GPIO_PDDR = 0x00;
    GPIO_PDDDR = 0xFF;
    GPIO_PEDR = 0x00;
    GPIO_PEDDR = 0x03;
    GPIO_PFDDR = 0x00;
    GPIO_PGDR = 0x00;
    GPIO_PGDDR = 0x05;
    GPIO_PHDR = 0x00;
    GPIO_PHDDR = 0x7C;
#else
    GPIO_PADR = 0x00;
    GPIO_PADDR = 0x00;
    GPIO_PBDR = 0x12;  /* RY_ON BBACKLIGHT */
    GPIO_PBDDR = 0x32; /* RY_ON BBACKLIGHT BREV */
    GPIO_PCDR = 0x00;
    GPIO_PCDDR = 0x00;
    GPIO_PDDR = 0x00;
    GPIO_PDDDR = 0x00;
    GPIO_PEDR = 0x00;
    GPIO_PEDDR = 0x03; /* RDLED, GRLED */
    GPIO_PFDDR = 0x00;
    GPIO_PGDR = 0x00;
    GPIO_PGDDR = 0x01; /* EECLOCK */
    GPIO_PHDR = 0x00;
    GPIO_PHDDR = 0x40; /* WatchDog */
#endif
    *(volatile unsigned *)EP9312_PWRCNT = EP9312_PWRCNT_UARTBAUD;

    /* Machine number, as defined in linux/arch/arm/tools/mach-types */
    gd->bd->bi_arch_number = MACH_TYPE_TB0319;

    /* adress of boot parameters */
    gd->bd->bi_boot_params = CONFIG_BOOT_PARAM_ADDR;
    return(0);
}

int misc_init_r()
{
	DECLARE_GLOBAL_DATA_PTR;
	struct rtc_time tm;
	int magic;
	char mac[6];

	i2c_init (CFG_I2C_SPEED, CFG_I2C_SLAVE);
	sflash_init();
	if ( eeprom_read (CFG_DEF_EEPROM_ADDR, 0x1000, (char*)&magic, sizeof(magic) ) == 0 &&
	     magic == 0x43414d45 )
	{
	    eeprom_read (CFG_DEF_EEPROM_ADDR, 0x1004, mac, sizeof(mac) );
	    memcpy (gd->bd->bi_enetaddr, mac, 6);
	} else {
	    printf( "Warning: eeprom does not have valid contents.\n" );
	}

#if (CONFIG_COMMANDS & CFG_CMD_DATE)
	rtc_get (&tm);
	*(volatile unsigned long *)0x8092000C = mktime(tm.tm_year, tm.tm_mon, tm.tm_mday,
						       tm.tm_hour, tm.tm_min, tm.tm_sec) + 1;
#endif
	return(0);
}

int dram_init (void)
{
	DECLARE_GLOBAL_DATA_PTR;

	gd->bd->bi_dram[0].start = PHYS_SDRAM_1;
	gd->bd->bi_dram[0].size = get_ram_size((void*)PHYS_SDRAM_1, PHYS_SDRAM_SIZE_1);
	gd->bd->bi_dram[1].start = PHYS_SDRAM_2;
	gd->bd->bi_dram[1].size = get_ram_size((void*)PHYS_SDRAM_2, PHYS_SDRAM_SIZE_2);

	return(0);
}

void hw_watchdog_reset(void)
{
	GPIO_PHDR ^= 0x40;
}
